<?php

namespace Drupal\cookiebot\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Path\AliasManager;

/**
 * Cookiebot settings form.
 */
class CookiebotForm extends ConfigFormBase {

  /**
   * Entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   *   The entity type manager.
   */
  protected $entityTypeManager;


  /**
   * Alias manager.
   *
   * @var \Drupal\Core\Path\AliasManager
   *   The Alias Manager.
   */
  protected $aliasManager;

  /**
   * Constructs a object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Entity\EntityTypeManager $entity_manager
   *   The entity type manager.
   * @param \Drupal\Core\Path\AliasManager $alias_manager
   *   The alias manager.
   */
  public function __construct(ConfigFactoryInterface $config_factory, EntityTypeManager $entity_manager, AliasManager $alias_manager) {
    parent::__construct($config_factory);
    $this->setConfigFactory($config_factory);
    $this->entityTypeManager = $entity_manager;
    $this->aliasManager = $alias_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('path.alias_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'cookiebot.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'cookiebot_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('cookiebot.settings');
    $form['cookiebot_cbid'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Your cookiebot Domain Group ID (CBID)'),
      '#description' => $this->t("This ID looks like 00000000-0000-0000-0000-000000000000. You can find it in the <a href='https://www.cookiebot.com/en/manage'>Cookiebot Manager</a> on the 'Your scripts' tab."),
      '#default_value' => $config->get('cookiebot_cbid'),
    ];

    $form['cookiebot_declaration'] = [
      '#type' => 'details',
      '#title' => $this->t('Cookie declaration'),
    ];

    $form['cookiebot_declaration']['cookiebot_show_declaration'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show the Cookiebot cookie declaration'),
      '#description' => $this->t('Automatically show the full Cookiebot cookie declaration on the given page.'),
      '#default_value' => $config->get('cookiebot_show_declaration'),
    ];

    $declaration_node_id = '';
    $alias = $this->aliasManager->getPathByAlias($config->get('cookiebot_show_declaration_node_id'));
    if (preg_match('/node\/(\d+)/', $alias, $matches)) {
      $declaration_node_id = $this->entityTypeManager->getStorage('node')->load($matches[1]);
    }

    $form['cookiebot_declaration']['cookiebot_show_declaration_node_id'] = [
      '#type' => 'entity_autocomplete',
      '#target_type' => 'node',
      '#title' => $this->t('Node page title'),
      '#description' => $this->t('Show the full cookie declaration declaration on the node page with the given title.'),
      '#default_value' => $declaration_node_id,
      '#states' => [
        'visible' => [
          ':input[name="cookiebot_show_declaration"]' => [
            'checked' => TRUE,
          ],
        ],
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $cbid_trimmed = trim($form_state->getValue('cookiebot_cbid'));
    $form_state->setValue('cookiebot_cbid', $cbid_trimmed);

    if (!preg_match('/^[0-9a-z]{8}-[0-9a-z]{4}-[0-9a-z]{4}-[0-9a-z]{4}-[0-9a-z]{12}$/', $cbid_trimmed)) {
      $form_state->setErrorByName('cookiebot_cbid', $this->t('The entered Domain Group ID is not formatted correctly.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    $alias = $this->aliasManager->getAliasByPath('/node/' . $form_state->getValue('cookiebot_show_declaration_node_id'));
    $this->config('cookiebot.settings')
      ->set('cookiebot_cbid', $form_state->getValue('cookiebot_cbid'))
      ->set('cookiebot_show_declaration', $form_state->getValue('cookiebot_show_declaration'))
      ->set('cookiebot_show_declaration_node_id', $alias)
      ->save();
  }

}
